﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using System.Collections;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Drawing;

namespace AdornmentLayerCSharp
{
    public class LegendAdornmentLayer : AdornmentLayer
    {
        private Collection<LayerOverlay> drawingOverlays;

        private float legendIconWidth;
        private float legendIconHeight;
        private float indent;

        public LegendAdornmentLayer()
            : this(0, 0, null)
        {
        }

        public LegendAdornmentLayer(float xOffsetOnScreen, float yOffsetOnScreen, IEnumerable<Overlay> drawingOverlays)
        {
            XOffsetInPixel = xOffsetOnScreen;
            YOffsetInPixel = yOffsetOnScreen;

            legendIconWidth = 15;
            legendIconHeight = 15;

            indent = 10;

            this.drawingOverlays = new Collection<LayerOverlay>();
            if (drawingOverlays != null)
            {
                foreach (LayerOverlay overlay in drawingOverlays)
                {
                    if (overlay is LayerOverlay)
                    {
                        this.drawingOverlays.Add(overlay);
                    }
                }
            }
        }

        public float LegendIconWidth
        {
            get { return legendIconWidth;}
            set { legendIconWidth = value; }
        }

        public float LegendIconHeight
        {
            get { return legendIconHeight; }
            set { legendIconHeight = value; }
        }

        public float Indent
        {
            get { return indent; }
            set { indent = value; }
        }

        public Collection<LayerOverlay> DrawingOverlays
        {
            get { return drawingOverlays; }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            float yOffsetInAll = YOffsetInPixel;

            DrawingFontStyles fontStyle = DrawingFontStyles.Regular;
            GeoFont font = new GeoFont("Arial", 10, fontStyle);

            for (int i = 0; i < drawingOverlays.Count; i++)
            {
                // Draw the overlay's name.
                DrawingRectangleF rectangle = canvas.MeasureText(drawingOverlays[i].Name, font);
                DrawText(canvas, XOffsetInPixel + rectangle.Width / 2, yOffsetInAll + rectangle.Height / 2, font, drawingOverlays[i].Name);

                // Update y offset.
                yOffsetInAll += (rectangle.Height + 2);

                for (int j = 0; j < drawingOverlays[i].Layers.Count; j++)
                {
                    FeatureLayer currentLayer = drawingOverlays[i].Layers[j] as FeatureLayer;
                    if (currentLayer != null)
                    {
                        // Calculate the x offset and y offset, then draw legend.
                        DrawLegend(canvas, currentLayer, XOffsetInPixel + indent + legendIconWidth / 2, yOffsetInAll + legendIconHeight / 2);

                        DrawingRectangleF layerRectangle = canvas.MeasureText(drawingOverlays[i].Layers[j].Name, font);
                        DrawText(canvas, XOffsetInPixel + legendIconWidth + indent + layerRectangle.Width / 2 + 5, yOffsetInAll + layerRectangle.Height / 2, font, drawingOverlays[i].Layers[j].Name);

                        // Update y offset.
                        yOffsetInAll += (rectangle.Height + 2);
                    }
                }
            }
        }

        private void DrawLegend(GeoCanvas canvas, FeatureLayer layer, float xOffset, float yOffset)
        {
            GdiPlusGeoCanvas legendCanvas = new GdiPlusGeoCanvas();
            Bitmap legendBitmap = null;
            try
            {
                legendBitmap = new Bitmap((int)legendIconWidth, (int)legendIconHeight);
                legendCanvas.BeginDrawing(legendBitmap, new RectangleShape(0, legendIconWidth, legendIconHeight, 0), GeographyUnit.DecimalDegree);
                layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.DrawSample(legendCanvas);
                layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.DrawSample(legendCanvas);
                layer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.DrawSample(legendCanvas);

                legendCanvas.Flush();
                if (canvas is GdiPlusGeoCanvas)
                {
                    ((GdiPlusGeoCanvas)canvas).DrawScreenImageWithoutScaling(legendBitmap, xOffset, yOffset, DrawingLevel.LevelOne, 0, 0, 0);
                }
            }
            finally
            {
                legendCanvas.EndDrawing();
                if (legendBitmap != null)
                {
                    legendBitmap.Dispose();
                    legendBitmap = null;
                }
            }
        }     

        private void DrawText(GeoCanvas canvas, float xOffsetInPixel, float yOffsetInPixel, GeoFont font, string text)
        {
            if (!string.IsNullOrEmpty(text))
            {
                canvas.DrawTextWithScreenCoordinate(text, font, new GeoSolidBrush(GeoColor.StandardColors.Blue), xOffsetInPixel, yOffsetInPixel, DrawingLevel.LevelThree);
            }
        }
    }
}
